Package: irace
Type: Package
Title: Iterated Racing for Automatic Algorithm Configuration
Description: Iterated race is an extension of the Iterated F-race method for
             the automatic configuration of optimization algorithms, that is,
             (offline) tuning their parameters by finding the most appropriate
             settings given a set of instances of an optimization problem.
             M. López-Ibáñez, J. Dubois-Lacoste, L. Pérez Cáceres, T. Stützle,
             and M. Birattari (2016) <doi:10.1016/j.orp.2016.09.002>.
Version: 4.4.3
Authors@R: c(person("Manuel", "López-Ibáñez", role = c("aut", "cre"),
                              email = "manuel.lopez-ibanez@manchester.ac.uk",
                              comment = c(ORCID = "0000-0001-9974-1295")),
             person("Jérémie", "Dubois-Lacoste", role = "aut"),
             person("Leslie", "Pérez Cáceres", role = "aut"),
             person("Thomas", "Stützle", role = "aut"),
             person("Mauro", "Birattari", role = "aut"),
             person("Eric", "Yuan", role = "ctb"),
             person("Prasanna", "Balaprakash", role = "ctb"),
             person("Nguyen", "Dang", role = "ctb", email = "nttd@st-andrews.ac.uk"))
Depends: R (>= 4.0.0)
Imports: R6, codetools, compiler, data.table (>= 1.15.0), fs,
        matrixStats (>= 1.4.1), spacefillr, stats, utils, withr
Suggests: Rmpi (>= 0.6.0), highr, knitr, parallel, testthat (>= 3.1.8)
VignetteBuilder: knitr
License: GPL (>= 2)
URL: https://mlopez-ibanez.github.io/irace/,
        https://github.com/MLopez-Ibanez/irace
BugReports: https://github.com/MLopez-Ibanez/irace/issues
ByteCompile: yes
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: GNU make
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-04-23 13:43:45 UTC; manu
Author: Manuel López-Ibáñez [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9974-1295>),
  Jérémie Dubois-Lacoste [aut],
  Leslie Pérez Cáceres [aut],
  Thomas Stützle [aut],
  Mauro Birattari [aut],
  Eric Yuan [ctb],
  Prasanna Balaprakash [ctb],
  Nguyen Dang [ctb]
Maintainer: Manuel López-Ibáñez <manuel.lopez-ibanez@manchester.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-23 14:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 20:27:06 UTC; unix
