# Generated by roxygen2: do not edit by hand

S3method(param_quantile,ParamCat)
S3method(param_quantile,ParamInt)
S3method(param_quantile,ParamOrd)
S3method(param_quantile,ParamReal)
S3method(print,ParameterSpace)
S3method(sample_model,ParamCat)
S3method(sample_model,ParamInt)
S3method(sample_model,ParamOrd)
S3method(sample_model,ParamReal)
S3method(sample_unif,ParamCat)
S3method(sample_unif,ParamInt)
S3method(sample_unif,ParamOrd)
S3method(sample_unif,ParamReal)
export(ablation)
export(ablation_cmdline)
export(buildCommandLine)
export(checkIraceScenario)
export(checkParameters)
export(checkScenario)
export(check_output_target_runner)
export(configurations_print)
export(configurations_print_command)
export(defaultScenario)
export(getConfigurationById)
export(getConfigurationByIteration)
export(getFinalElites)
export(get_instanceID_seed_pairs)
export(get_random_seed)
export(has_testing_data)
export(irace)
export(irace.cmdline)
export(irace_cmdline)
export(irace_license)
export(irace_main)
export(irace_summarise)
export(irace_version)
export(multi_irace)
export(param_cat)
export(param_int)
export(param_ord)
export(param_real)
export(parametersNew)
export(path_rel2abs)
export(plotAblation)
export(printParameters)
export(printScenario)
export(psRace)
export(readConfigurationsFile)
export(readParameters)
export(readScenario)
export(read_ablogfile)
export(read_logfile)
export(read_pcs_file)
export(removeConfigurationsMetaData)
export(restore_random_seed)
export(scenario_update_paths)
export(set_random_seed)
export(target_evaluator_default)
export(target_runner_default)
export(testConfigurations)
export(testing_fromfile)
export(testing_fromlog)
import(data.table)
import(matrixStats)
import(stats)
import(withr)
importFrom(R6,R6Class)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,boxplot)
importFrom(graphics,bxp)
importFrom(graphics,grid)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,strwidth)
importFrom(spacefillr,generate_sobol_set)
importFrom(utils,str)
