Example configuration scenario: Spear
=====================================

This example scenario was inspired by ParamILS. The Spear program can
be downloaded from: 

 http://www.cs.ubc.ca/labs/beta/Projects/ParamILS/algorithms.html

WARNING: Please note that no source code is available and we do not know what
this executable is actually doing. We do NOT recommend running this executable
in your computer.  If Spear doesn't work in your computer, there is nothing we
can do to fix it.  Unless you absolutely must run Spear, we suggest to use a
different scenario for which source code is available.

* Tuning instances:

 http://www.cs.ubc.ca/labs/beta/Projects/ParamILS/benchmark_instances/SpearSWV/SWV-scrambled-first302.tar.gz

* Testing instances:

 http://www.cs.ubc.ca/labs/beta/Projects/ParamILS/benchmark_instances/SpearSWV/SWV-scrambled-last302.tar.gz


Two parameter files are provided: 

 * parameters-cat.txt only contains categorical parameters.

 * parameters-mixed.txt contains a mix of numerical and categorical parameters.

