Example configuration scenario: MOACO
=====================================

This example scenario shows how irace can be used to tune a
multi-objective optimization framework. It uses the multi-objective
ant colony optimization (MOACO) framework proposed here for the bi-objective
traveling salesperson problem (bTSP):

    Manuel López-Ibáñez and Thomas Stützle. The Automatic Design of
    Multi-Objective Ant Colony Optimization Algorithms. IEEE
    Transactions on Evolutionary Computation, 2012. 
    doi:10.1109/TEVC.2011.2182651
    (PDF preprint) https://iridia.ulb.ac.be/IridiaTrSeries/link/IridiaTr2011-003.pdf

The MOACO framework software can be download from:

    http://lopez-ibanez.eu/moaco

Instances of the bTSP can be generated by concatenating two single-objective TSP
instances:

    $ cat euclidA100.tsp euclidB100.tsp > euclidAB100.tsp

TSP instances and more information is available at:

https://iridia.ulb.ac.be/supp/IridiaSupp2016-003/index.html



