Additional examples of scenarios can be found in:

Capping Methods for the Automatic Configuration of Optimization Algorithms
Marcelo de Souza, Marcus Ritt and Manuel López-Ibáñez
https://github.com/souzamarcelo/supp-cor-capopt

 * ACOTSP: ant colony optimization algorithms for the symmetric traveling salesperson problem.
 * HEACOL: hybrid evolutionary algorithm for graph coloring.
 * TSBPP: tabu search for the bin packing problem.
 * HHBQP: hybrid heuristic for unconstrained binary quadratic programming.
 * LKH: a heuristic algorithm for solving the symmetric traveling salesperson problem.
 * SCIP: an exact solver for mixed integer programs for solving the
   combinatorial auction winner determination problem.
   

http://aclib.net/
Frank Hutter, Manuel López-Ibáñez, Chris Fawcett, Marius Thomas Lindauer,
Holger H. Hoos, Kevin Leyton-Brown, and Thomas Stützle. AClib: a Benchmark
Library for Algorithm Configuration. In P. M. Pardalos, M. G. C. Resende,
C. Vogiatzis, and J. L. Walteros, editors, Learning and Intelligent
Optimization, 8th International Conference, LION 8, volume 8426 of Lecture
Notes in Computer Science, pages 36–40. Springer, 2014. 
